# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin.common import waiters
from designate_tempest_plugin.tests import base
from designate_tempest_plugin import data_utils as dns_data_utils

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseZonesImportTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'version', 'links',
                     'status', 'message', 'zone_id']

    @classmethod
    def setup_clients(cls):
        super(BaseZonesImportTest, cls).setup_clients()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(BaseZonesImportTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="BaseZonesImportTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BaseZonesImportTest, cls).resource_cleanup()


class ZonesImportTest(BaseZonesImportTest):
    credentials = ["primary", "admin", "alt",
                   "project_member", "project_reader"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZonesImportTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ZonesImportTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.ZoneImportsClient()
        cls.client = cls.os_primary.dns_v2.ZoneImportsClient()
        cls.alt_client = cls.os_alt.dns_v2.ZoneImportsClient()

    def clean_up_resources(self, zone_import_id):
        zone_import = self.client.show_zone_import(zone_import_id)[1]
        if zone_import['zone_id']:  # A zone was actually created.
            waiters.wait_for_zone_import_status(
                self.client, zone_import_id, const.COMPLETE)
            self.client.delete_zone_import(zone_import['id'])
            self.wait_zone_delete(self.zones_client, zone_import['zone_id'])
        else:  # Import has failed and zone wasn't created.
            self.client.delete_zone_import(zone_import['id'])

    @decorators.idempotent_id('2e2d907d-0609-405b-9c96-3cb2b87e3dce')
    def test_create_zone_import(self):
        LOG.info('Create a zone import')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_zone_import", suffix=self.tld_name)
        zone_data = dns_data_utils.rand_zonefile_data(name=zone_name)
        zone_import = self.client.create_zone_import(
            zonefile_data=zone_data)[1]
        self.addCleanup(self.clean_up_resources, zone_import['id'])
        # Make sure we complete the import and have the zone_id for cleanup
        waiters.wait_for_zone_import_status(
            self.client, zone_import['id'], const.COMPLETE)

        # Test with no extra header overrides (sudo-project-id)
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')

        self.check_CUD_RBAC_enforcement(
            'ZoneImportsClient', 'create_zone_import', expected_allowed, False)

    @decorators.idempotent_id('31eaf25a-9532-11eb-a55d-74e5f9e2a801')
    def test_create_zone_import_invalid_ttl(self):
        LOG.info('Try to create a zone import using invalid TTL value')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_zone_import_invalid_ttl", suffix=self.tld_name)
        zone_data = dns_data_utils.rand_zonefile_data(name=zone_name,
                                                      ttl='zahlabut')
        zone_import = self.client.create_zone_import(
            zonefile_data=zone_data)[1]
        self.addCleanup(self.clean_up_resources, zone_import['id'])
        waiters.wait_for_zone_import_status(
            self.client, zone_import['id'], "ERROR")

    @decorators.idempotent_id('31eaf25a-9532-11eb-a55d-74e5f9e2a801')
    def test_create_zone_import_invalid_name(self):
        LOG.info('Try to create a zone import using invalid name')
        zone_import = self.client.create_zone_import(
            zonefile_data=dns_data_utils.rand_zonefile_data(name='@@@'))[1]
        self.addCleanup(self.clean_up_resources, zone_import['id'])
        waiters.wait_for_zone_import_status(
            self.client, zone_import['id'], "ERROR")

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('c8909558-0dc6-478a-9e91-eb97b52e59e0')
    def test_show_zone_import(self):
        LOG.info('Create a zone import')
        zone_name = dns_data_utils.rand_zone_name(
            name="show_zone_import", suffix=self.tld_name)
        zone_data = dns_data_utils.rand_zonefile_data(name=zone_name)
        zone_import = self.client.create_zone_import(
            zonefile_data=zone_data)[1]
        self.addCleanup(self.clean_up_resources, zone_import['id'])
        # Make sure we complete the import and have the zone_id for cleanup
        waiters.wait_for_zone_import_status(
            self.client, zone_import['id'], const.COMPLETE)

        LOG.info('Re-Fetch the zone import')
        resp, body = self.client.show_zone_import(zone_import['id'])

        LOG.info('Ensure the fetched response matches the expected one')
        self.assertExpected(zone_import, body, self.excluded_keys)

        # Test with no extra header overrides (all_projects, sudo-project-id)
        expected_allowed = ['os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member',
                                     'os_project_reader'])

        self.check_list_show_RBAC_enforcement(
            'ZoneImportsClient', 'show_zone_import', expected_allowed, True,
            zone_import['id'])

        # Test with x-auth-all-projects
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'ZoneImportsClient', 'show_zone_import', expected_allowed, False,
            zone_import['id'], headers=self.all_projects_header)

    @decorators.idempotent_id('56a16e68-b241-4e41-bc5c-c40747fa68e3')
    def test_delete_zone_import(self):
        LOG.info('Create a zone import')
        zone_name = dns_data_utils.rand_zone_name(
            name="delete_zone_import", suffix=self.tld_name)
        zone_data = dns_data_utils.rand_zonefile_data(name=zone_name)
        zone_import = self.client.create_zone_import(
            zonefile_data=zone_data)[1]
        waiters.wait_for_zone_import_status(self.client, zone_import['id'],
                                            const.COMPLETE)
        zone_import = self.client.show_zone_import(zone_import['id'])[1]
        self.addCleanup(self.wait_zone_delete,
                        self.zones_client,
                        zone_import['zone_id'])

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement(
            'ZoneImportsClient', 'delete_zone_import', expected_allowed, True,
            zone_import['id'])

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement(
            'ZoneImportsClient', 'delete_zone_import', expected_allowed, False,
            zone_import['id'], headers=self.all_projects_header)
        self.check_CUD_RBAC_enforcement(
            'ZoneImportsClient', 'delete_zone_import', expected_allowed, False,
            zone_import['id'],
            headers={'x-auth-sudo-project-id': self.client.project_id})

        LOG.info('Delete the zone')
        resp, body = self.client.delete_zone_import(zone_import['id'])

        LOG.info('Ensure successful deletion of imported zones')
        self.assertRaises(lib_exc.NotFound,
            lambda: self.client.show_zone_import(zone_import['id']))

    @decorators.idempotent_id('9eab76af-1995-485f-a2ef-8290c1863aba')
    def test_list_zones_imports(self):
        LOG.info('Create a zone import')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_zone_imports", suffix=self.tld_name)
        zone_data = dns_data_utils.rand_zonefile_data(name=zone_name)
        zone_import = self.client.create_zone_import(
            zonefile_data=zone_data)[1]
        self.addCleanup(self.clean_up_resources, zone_import['id'])
        # Make sure we complete the import and have the zone_id for cleanup
        waiters.wait_for_zone_import_status(
            self.client, zone_import['id'], const.COMPLETE)

        LOG.info('List zones imports')
        body = self.client.list_zone_imports()[1]

        self.assertGreater(len(body['imports']), 0)

        # Test RBAC - Users that are allowed to call list, but should get
        #             zero zones.
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed = ['os_admin']
        else:
            expected_allowed = ['os_alt']

        self.check_list_RBAC_enforcement_count(
            'ZoneImportsClient', 'list_zone_imports', expected_allowed, 0)

        # Test that users who should see the zone, can see it.
        expected_allowed = ['os_primary']

        self.check_list_IDs_RBAC_enforcement(
            'ZoneImportsClient', 'list_zone_imports', expected_allowed,
            [zone_import['id']])

        # Test RBAC with x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'ZoneImportsClient', 'list_zone_imports', expected_allowed,
            [zone_import['id']],
            headers={'x-auth-sudo-project-id': self.client.project_id})

    @decorators.idempotent_id('2c1fa20e-9554-11eb-a55d-74e5f9e2a801')
    def test_show_import_impersonate_another_project(self):

        LOG.info('Import zone "A" using primary client')
        zone_name = dns_data_utils.rand_zone_name(
            name="show_zone_import_impersonate", suffix=self.tld_name)
        zone_data = dns_data_utils.rand_zonefile_data(name=zone_name)
        zone_import = self.client.create_zone_import(
            zonefile_data=zone_data)[1]
        self.addCleanup(self.clean_up_resources, zone_import['id'])

        # Make sure we complete the import and have the zone_id for cleanup
        waiters.wait_for_zone_import_status(
            self.client, zone_import['id'], const.COMPLETE)

        LOG.info('Show a zone import for a Primary tenant, using Alt tenant. '
                 'Expected:404 NotFound')
        self.assertRaises(lib_exc.NotFound,
                          lambda: self.alt_client.show_zone_import(
                              zone_import['id']))

        LOG.info('Show a zone import for a Primary tenant using Alt tenant '
                 'and "x-auth-sudo-project-id" HTTP header. '
                 'Expected:403 Forbidden')
        self.assertRaises(
            lib_exc.Forbidden,
            lambda: self.alt_client.show_zone_import(
                zone_import['id'],
                headers={'x-auth-sudo-project-id': zone_import[
                    'project_id']}))

        LOG.info('Show a zone import for a Primary tenant, using Admin '
                 'tenant and "x-auth-sudo-project-id" HTTP header.')
        resp_body = self.admin_client.show_zone_import(uuid=None, headers={
                'x-auth-sudo-project-id': zone_import['project_id']})[1]

        LOG.info('Show a zone import for a Primary tenant, using Admin '
                 'tenant without "x-auth-sudo-project-id" HTTP header. '
                 'Expected:404 NotFound')
        self.assertRaises(
            lib_exc.NotFound, lambda: self.admin_client.show_zone_import(
                zone_import['id']))

        LOG.info('Ensure that the shown response matches the expected one')
        self.assertExpected(
            zone_import, resp_body['imports'][0], self.excluded_keys)

        # Test with x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'ZoneImportsClient', 'show_zone_import', expected_allowed, False,
            zone_import['id'],
            headers={'x-auth-sudo-project-id': self.client.project_id})

    @decorators.idempotent_id('7bd06ec6-9556-11eb-a55d-74e5f9e2a801')
    def test_list_import_zones_all_projects(self):
        LOG.info('Create import zone "A" using primary client')
        zone_name = dns_data_utils.rand_zone_name(
            name="_zone_imports_all_projects", suffix=self.tld_name)
        zone_data = dns_data_utils.rand_zonefile_data(name=zone_name)
        zone_import = self.client.create_zone_import(
            zonefile_data=zone_data)[1]
        self.addCleanup(self.clean_up_resources, zone_import['id'])
        # Make sure we complete the import and have the zone_id for cleanup
        waiters.wait_for_zone_import_status(
            self.client, zone_import['id'], const.COMPLETE)

        LOG.info('As Alt user list import zones for a Primary tenant, '
                 'using "x-auth-sudo-project-id" HTTP header. '
                 'Expected: 403 Forbidden')
        self.assertRaises(
            lib_exc.Forbidden, lambda: self.alt_client.list_zone_imports(
                headers={
                    'x-auth-sudo-project-id': zone_import['project_id']}))

        LOG.info('As Alt tenant list zone imports for all projects, using '
                 '"x-auth-all-projects" HTTP header, Expected: 403 Forbidden')
        self.assertRaises(
            lib_exc.Forbidden, lambda: self.alt_client.list_zone_imports(
                headers=self.all_projects_header))

        LOG.info('As Admin tenant list import zones for all projects')
        # Note: This is an all-projects list call, so other tests running
        #       in parallel will impact the list result set. Since the default
        #       pagination limit is only 20, we set a param limit of 1000 here.
        body = self.admin_client.list_zone_imports(
            headers=self.all_projects_header,
            params={'limit': 1000})[1]['imports']

        LOG.info('Ensure the fetched response includes previously '
                 'created import ID')
        listed_zone_import_ids = [item['id'] for item in body]
        self.assertIn(
            zone_import['id'], listed_zone_import_ids,
            "Failed, expected import ID:{} wasn't found in "
            "listed import IDs".format(zone_import['id']))

        # Test RBAC with x-auth-all-projects
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'ZoneImportsClient', 'list_zone_imports', expected_allowed,
            [zone_import['id']], headers=self.all_projects_header)


class ZonesImportTestNegative(BaseZonesImportTest):
    credentials = ["primary", "admin"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZonesImportTestNegative, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ZonesImportTestNegative, cls).setup_clients()
        cls.client = cls.os_primary.dns_v2.ZoneImportsClient()

    def _clean_up_resources(self, zone_import_id):
        zone_import = self.client.show_zone_import(zone_import_id)[1]
        if zone_import['zone_id']:  # A zone was actually created.
            waiters.wait_for_zone_import_status(
                self.client, zone_import_id, const.COMPLETE)
            self.client.delete_zone_import(zone_import['id'])
            self.wait_zone_delete(self.zones_client, zone_import['zone_id'])
        else:  # Import has failed and zone wasn't created.
            self.client.delete_zone_import(zone_import['id'])

    @decorators.idempotent_id('31eaf25a-9532-11eb-a55d-74e5f9e2a801')
    def test_create_zone_import_invalid_ttl(self):
        LOG.info('Try to create a zone import using invalid TTL value')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_zone_import_invalid_ttl", suffix=self.tld_name)
        zone_data = dns_data_utils.rand_zonefile_data(name=zone_name,
                                                      ttl='zahlabut')
        zone_import = self.client.create_zone_import(
            zonefile_data=zone_data, wait_until=const.ERROR)[1]
        self.addCleanup(self._clean_up_resources, zone_import['id'])

    @decorators.idempotent_id('31eaf25a-9532-11eb-a55d-74e5f9e2a801')
    def test_create_zone_import_invalid_name(self):
        LOG.info('Try to create a zone import using invalid name')
        zone_import = self.client.create_zone_import(
            zonefile_data=dns_data_utils.rand_zonefile_data(
                name='@@@'), wait_until=const.ERROR)[1]
        self.addCleanup(self._clean_up_resources, zone_import['id'])

    @decorators.idempotent_id('8fd744d2-9dff-11ec-9fb6-201e8823901f')
    def test_create_zone_import_invalid_file_data(self):
        LOG.info('Try to create a zone import using random generated'
                 ' import file data')
        zone_file_data = dns_data_utils.rand_string(size=100)
        zone_import = self.client.create_zone_import(zone_file_data)[1]
        self.addCleanup(self.client.delete_zone_import, zone_import['id'])
        waiters.wait_for_zone_import_status(
            self.client, zone_import['id'], const.ERROR)

    @decorators.idempotent_id('4fb9494e-9e23-11ec-8378-201e8823901f')
    def test_zone_cannot_be_update_by_import(self):
        LOG.info('Create a Zone named: "...zone_to_update..."')
        zone_name = dns_data_utils.rand_zone_name(
            name='zone_to_update', suffix=self.tld_name)
        zone = self.zones_client.create_zone(
            name=zone_name, wait_until=const.ACTIVE)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        LOG.info('Use zone import to update an existing zone, expected: zone'
                 ' import gets into the ERROR status ')
        zone_import_data = dns_data_utils.rand_zonefile_data(name=zone_name)
        zone_import = self.client.create_zone_import(zone_import_data)[1]
        waiters.wait_for_zone_import_status(
            self.client, zone_import['id'], const.ERROR)
        self.addCleanup(self._clean_up_resources, zone_import['id'])

    @decorators.idempotent_id('5fa8016e-6ed1-11ec-9bd7-201e8823901f')
    def test_create_zone_import_invalid_content_type(self):
        LOG.info('Try to create a zone import using: "Content-Type:Zahlabut"'
                 ' HTTP header in POST request')
        with self.assertRaisesDns(
                lib_exc.InvalidContentType, 'unsupported_content_type', 415):
            self.client.create_zone_import(
                headers={'Content-Type': 'Zahlabut'})



# Copyright 2020 Cloudification GmbH. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from designate_tempest_plugin.services.dns.v2.json import base


class SharedZonesClient(base.DnsClientV2Base):

    @base.handle_errors
    def create_zone_share(self, zone_id, target_project_id, headers=None):
        """Create a new zone share for a project ID.

        :param zone_id: Zone UUID to share
        :param target_project_id: Project ID that will gain access to specified
                                  zone
        :param headers: (dict): The headers to use for the request.
        :return: Zone share dict
        """
        resp, body = self._create_request(
            'zones/{}/shares'.format(zone_id),
            data={'target_project_id': target_project_id}, headers=headers,
            extra_headers=True)

        # Endpoint should Return a HTTP 201
        self.expected_success(201, resp.status)

        return resp, body

    @base.handle_errors
    def show_zone_share(self, zone_id, zone_share_id, headers=None):
        """Get the zone share object

        :param zone_id: Zone UUID for the share
        :param zone_share_id: The zone share ID
        :param headers: (dict): The headers to use for the request.
        :return: Zone share dict
        """
        return self._show_request('zones/{}/shares'.format(zone_id),
                                  zone_share_id, headers=headers)

    @base.handle_errors
    def list_zone_shares(self, zone_id, params=None, headers=None):
        """List zone shares

        :param zone_id: Zone UUID to query for the shares
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param headers: (dict): The headers to use for the request.
        :return: Zone shares list.
        """
        return self._list_request('zones/{}/shares'.format(zone_id),
                                  params=params, headers=headers)

    @base.handle_errors
    def delete_zone_share(self, zone_id, zone_share_id, headers=None):
        """Deletes the zone share

        :param zone_id: Zone UUID for the share
        :param zone_share_id: The zone share ID
        :param headers: (dict): The headers to use for the request.
        :return: None
        """
        resp, body = self._delete_request('zones/{}/shares'.format(zone_id),
                                          zone_share_id, headers=headers)

        # Endpoint should Return a HTTP 204 - No Content
        self.expected_success(204, resp.status)

        return resp, body



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools

from designate_tempest_plugin.tests import base
from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin.common import waiters
from designate_tempest_plugin import data_utils as dns_data_utils

import tempest.api.network.base

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BasePtrTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'version', 'links',
                     'status', 'action']

    @classmethod
    def setup_clients(cls):
        super(BasePtrTest, cls).setup_clients()

        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.admin_network_client = cls.os_admin.networks_client
        cls.admin_subnet_client = cls.os_admin.subnets_client

    @classmethod
    def resource_setup(cls):
        super(BasePtrTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name='BasePtrTest')
        cls.tld_name = tld_name[:-1]
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

        # Create dedicated External network for PTRs tests
        network_name = data_utils.rand_name('designate-tempest-fips-external')
        cls.external_network = cls.admin_network_client.create_network(
            name=network_name, **{'router:external': True})['network']
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.admin_network_client.delete_network,
            cls.external_network['id'])

        # Create subnet for External network
        cls.external_subnet = cls.admin_subnet_client.create_subnet(
            network_id=cls.external_network['id'],
            allocation_pools=[
                {'start': '198.51.100.10', 'end': '198.51.100.200'}],
            cidr='198.51.100.0/24', ip_version=4, enable_dhcp=False)
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.admin_subnet_client.delete_subnet,
            cls.external_subnet['subnet']['id'])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BasePtrTest, cls).resource_cleanup()


class DesignatePtrRecord(BasePtrTest, tempest.test.BaseTestCase):

    credentials = ['primary', 'admin']

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(DesignatePtrRecord, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(DesignatePtrRecord, cls).setup_clients()
        cls.admin_ptr_client = cls.os_admin.dns_v2.PtrClient()
        cls.primary_ptr_client = cls.os_primary.dns_v2.PtrClient()
        cls.primary_floating_ip_client = cls.os_primary.floating_ips_client

    @classmethod
    def resource_setup(cls):
        super(DesignatePtrRecord, cls).resource_setup()

        # The 'arpa' TLD is a special case as the negative test class also
        # needs to use this space. To stop test class concurrency conflicts,
        # let each class manage different TLDs for the reverse namespace.
        cls.arpa_tld = cls.admin_tld_client.create_tld(tld_name='arpa')

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.arpa_tld[1]['id'])
        super(DesignatePtrRecord, cls).resource_cleanup()

    def _set_ptr(self, ptr_name=None, ttl=None, description=None,
                 headers=None, tld=None, fip_id=None):
        if not tld:
            tld = self.tld_name
        if not fip_id:
            fip = self.primary_floating_ip_client.create_floatingip(
                floating_network_id=self.external_network['id'])[
                'floatingip']
            fip_id = fip['id']
            self.addCleanup(
                self.primary_floating_ip_client.delete_floatingip, fip_id)
        ptr = self.primary_ptr_client.set_ptr_record(
            fip_id, ptr_name=ptr_name, ttl=ttl, description=description,
            headers=headers, tld=tld)
        self.addCleanup(self.unset_ptr, self.primary_ptr_client, fip_id)

        self.assertEqual('CREATE', ptr['action'])
        self.assertEqual('PENDING', ptr['status'])
        waiters.wait_for_ptr_status(
            self.primary_ptr_client, fip_id=fip_id, status=const.ACTIVE)
        return fip_id, ptr

    def _unset_ptr(self, fip_id):
        self.primary_ptr_client.unset_ptr_record(fip_id)
        waiters.wait_for_ptr_status(
            self.primary_ptr_client, fip_id=fip_id, status=const.INACTIVE)

    @decorators.idempotent_id('2fb9d6ea-871d-11eb-9f9a-74e5f9e2a801')
    def test_set_floatingip_ptr(self):
        self._set_ptr()

    @decorators.idempotent_id('9179325a-87d0-11eb-9f9a-74e5f9e2a801')
    def test_show_floatingip_ptr(self):
        fip_id, ptr = self._set_ptr()
        show_ptr = self.primary_ptr_client.show_ptr_record(
            floatingip_id=fip_id)
        self.assertExpected(ptr, show_ptr, self.excluded_keys)

    @decorators.idempotent_id('d3128a92-e3bd-11eb-a097-74e5f9e2a801')
    def test_show_floatingip_ptr_impersonate_another_project(self):
        fip_id, ptr = self._set_ptr()

        LOG.info('As Admin user, show PTR record created by Primary'
                 ' user by including "x-auth-sudo-project-id" HTTP header'
                 ' in HTTP request.')
        show_ptr = self.admin_ptr_client.show_ptr_record(
            floatingip_id=fip_id,
            headers={
                'x-auth-sudo-project-id': self.primary_ptr_client.project_id})
        self.assertExpected(ptr, show_ptr, self.excluded_keys)

    @decorators.idempotent_id('9187a9c6-87d4-11eb-9f9a-74e5f9e2a801')
    def test_list_floatingip_ptr_records(self):
        number_of_ptr_records = 3
        created_ptr_ids = []
        for _ in range(number_of_ptr_records):
            fip_id, ptr = self._set_ptr()
            created_ptr_ids.append(ptr['id'])
        received_ptr_ids = sorted(
            [item['id'] for item in
             self.primary_ptr_client.list_ptr_records()])
        self.assertEqual(
            sorted(created_ptr_ids), received_ptr_ids,
            'Failed - received PTR IDs: {} are not as'
            ' expected: {}'.format(created_ptr_ids, received_ptr_ids))

    @decorators.idempotent_id('a108d6f2-e3c0-11eb-a097-74e5f9e2a801')
    @decorators.skip_because(bug="1935977")
    def test_list_floatingip_ptr_all_projects(self):
        ptr = self._set_ptr()[1]
        LOG.info('Created PTR is:{}'.format(ptr))

        LOG.info('As Admin user, try to list PTR record for all projects '
                 'by including "x-auth-all-projects" HTTP header.')
        received_ptr_ids = [
            item['id'] for item in self.admin_ptr_client.list_ptr_records(
                headers={'x-auth-all-projects': True})]
        self.assertGreater(
            len(received_ptr_ids), 0,
            'Failed, "received_ptr_ids" should not be empty')
        self.assertIn(
            ptr['id'], received_ptr_ids,
            'Failed, expected ID was not found in "received_ptr_ids" list.')

    @decorators.idempotent_id('499b5a7e-87e1-11eb-b412-74e5f9e2a801')
    @testtools.skipUnless(config.CONF.dns_feature_enabled.bug_1932026_fixed,
                          'Skip unless bug 1932026 has been fixed.')
    def test_unset_floatingip_ptr(self):
        fip_id, ptr = self._set_ptr()
        self._unset_ptr(fip_id)


class DesignatePtrRecordNegative(BasePtrTest, tempest.test.BaseTestCase):

    credentials = ['primary', 'admin']

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(DesignatePtrRecordNegative, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(DesignatePtrRecordNegative, cls).setup_clients()
        cls.primary_ptr_client = cls.os_primary.dns_v2.PtrClient()
        cls.primary_floating_ip_client = cls.os_primary.floating_ips_client
        cls.admin_ptr_client = cls.os_admin.dns_v2.PtrClient()

    @classmethod
    def resource_setup(cls):
        super(DesignatePtrRecordNegative, cls).resource_setup()

        # The 'arpa' TLD is a special case as the positive test class also
        # needs to use this space. To stop test class concurrency conflicts,
        # let each class manage different TLDs for the reverse namespace.
        cls.in_addr_arpa_tld = cls.admin_tld_client.create_tld(
            tld_name='in-addr.arpa')

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.in_addr_arpa_tld[1]['id'])
        super(DesignatePtrRecordNegative, cls).resource_cleanup()

    def _set_ptr(self, ptr_name=None, ttl=None, description=None,
                 headers=None, tld=None, fip_id=None):
        if not tld:
            tld = self.tld_name
        if not fip_id:
            fip = self.primary_floating_ip_client.create_floatingip(
                floating_network_id=self.external_network['id'])[
                'floatingip']
            fip_id = fip['id']
            self.addCleanup(
                self.primary_floating_ip_client.delete_floatingip, fip_id)
        ptr = self.primary_ptr_client.set_ptr_record(
            fip_id, ptr_name=ptr_name, ttl=ttl, description=description,
            headers=headers, tld=tld)
        self.addCleanup(self.unset_ptr, self.primary_ptr_client, fip_id)
        self.assertEqual('CREATE', ptr['action'])
        self.assertEqual('PENDING', ptr['status'])
        waiters.wait_for_ptr_status(
            self.primary_ptr_client, fip_id=fip_id, status=const.ACTIVE)
        return fip_id, ptr

    @decorators.attr(type='negative')
    @decorators.idempotent_id('8392db50-cdd0-11eb-a00f-74e5f9e2a801')
    def test_set_floatingip_ptr_invalid_ttl(self):
        LOG.info('Try to set PTR record using invalid TTL value')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_object', 400):
            self._set_ptr(ttl=-10)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('0c9349ae-e2e8-11eb-a097-74e5f9e2a801')
    def test_set_floatingip_ptr_not_existing_fip_id(self):
        LOG.info('Try to set PTR record using not existing Floating IP')
        with self.assertRaisesDns(lib_exc.NotFound, 'not_found', 404):
            self._set_ptr(fip_id=data_utils.rand_uuid())

    @decorators.attr(type='negative')
    @decorators.idempotent_id('df217902-e3b2-11eb-a097-74e5f9e2a801')
    def test_set_floatingip_ptr_huge_size_description(self):
        LOG.info('Try to set PTR record using huge size description string')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_object', 400):
            self._set_ptr(description=dns_data_utils.rand_string(5000))

    @decorators.attr(type='negative')
    @decorators.idempotent_id('cb2264e2-e3b3-11eb-a097-74e5f9e2a801')
    def test_set_floatingip_ptr_invalid_name(self):
        invalid_names = ['', '@!(*&', 4564, dns_data_utils.rand_string(5000)]
        for name in invalid_names:
            LOG.info('Set PTR record using invalid name:{}'.format(name))
            with self.assertRaisesDns(
                    lib_exc.BadRequest, 'invalid_object', 400):
                self._set_ptr(description=dns_data_utils.rand_string(5000))

    @decorators.attr(type='negative')
    @decorators.idempotent_id('f616d216-51ac-11ec-8edf-201e8823901f')
    def test_show_floatingip_ptr_impersonate_another_project_no_header(self):
        fip_id, ptr = self._set_ptr()

        LOG.info('As Admin user, show PTR record created by Primary'
                 ' user, without including "x-auth-sudo-project-id" '
                 'HTTP header in request.')
        with self.assertRaisesDns(lib_exc.NotFound, 'not_found', 404):
            self.admin_ptr_client.show_ptr_record(floatingip_id=fip_id)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('0d132ff0-51ad-11ec-8edf-201e8823901f')
    @decorators.skip_because(bug="1935977")
    def test_list_floatingip_ptr_all_projects_no_header(self):
        ptr = self._set_ptr()[1]
        LOG.info('Created PTR is:{}'.format(ptr))

        LOG.info('As Admin user, try to list PTR record for all projects '
                 'without including "x-auth-all-projects" HTTP header.')
        received_ptr_ids = [
            item['id'] for item in self.admin_ptr_client.list_ptr_records()]
        self.assertEqual([], received_ptr_ids,
                         'Failed, "received_ptr_ids" list should be empty')



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_log import log as logging
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin.tests import base
from designate_tempest_plugin import data_utils as dns_data_utils

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseTransferRequestTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'key', 'links']

    @classmethod
    def setup_clients(cls):
        super(BaseTransferRequestTest, cls).setup_clients()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(BaseTransferRequestTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(
            name="BaseTransferRequestTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BaseTransferRequestTest, cls).resource_cleanup()


class TransferRequestTest(BaseTransferRequestTest):
    credentials = ["primary", "alt", "admin",
                   "project_member", "project_reader"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TransferRequestTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TransferRequestTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.TransferRequestClient()
        cls.alt_zone_client = cls.os_alt.dns_v2.ZonesClient()
        cls.client = cls.os_primary.dns_v2.TransferRequestClient()
        cls.alt_client = cls.os_alt.dns_v2.TransferRequestClient()

    @decorators.idempotent_id('2381d489-ad84-403d-b0a2-8b77e4e966bf')
    def test_create_transfer_request(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_transfer_request", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')

        self.check_CUD_RBAC_enforcement(
            'TransferRequestClient', 'create_transfer_request',
            expected_allowed, True, zone['id'])

        LOG.info('Create a zone transfer_request')
        transfer_request = self.client.create_transfer_request(zone['id'])[1]
        self.addCleanup(self.client.delete_transfer_request,
                        transfer_request['id'])

        LOG.info('Ensure we respond with ACTIVE status')
        self.assertEqual('ACTIVE', transfer_request['status'])

    @decorators.idempotent_id('5deae1ac-7c14-42dc-b14e-4e4b2725beb7')
    def test_create_transfer_request_scoped(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_transfer_request_scoped", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        transfer_request_data = dns_data_utils.rand_transfer_request_data(
            target_project_id=self.os_alt.credentials.project_id)

        LOG.info('Create a scoped zone transfer_request')
        transfer_request = self.client.create_transfer_request(
            zone['id'], transfer_request_data)[1]
        self.addCleanup(self.client.delete_transfer_request,
                        transfer_request['id'])

        LOG.info('Ensure we respond with ACTIVE status')
        self.assertEqual('ACTIVE', transfer_request['status'])

    @decorators.idempotent_id('4505152f-0a9c-4f02-b385-2216c914a0be')
    def test_create_transfer_request_empty_body(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_transfer_request_empty", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        LOG.info('Create a zone transfer_request')
        transfer_request = self.client.create_transfer_request_empty_body(
            zone['id'])[1]
        self.addCleanup(self.client.delete_transfer_request,
                        transfer_request['id'])

        LOG.info('Ensure we respond with ACTIVE status')
        self.assertEqual('ACTIVE', transfer_request['status'])

    @decorators.idempotent_id('64a7be9f-8371-4ce1-a242-c1190de7c985')
    def test_show_transfer_request(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="show_transfer_request", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone transfer_request')
        transfer_request = self.client.create_transfer_request(zone['id'])[1]
        self.addCleanup(self.client.delete_transfer_request,
                        transfer_request['id'])

        LOG.info('Fetch the transfer_request')
        body = self.client.show_transfer_request(transfer_request['id'])[1]

        LOG.info('Ensure the fetched response matches the '
                 'created transfer_request')
        self.assertExpected(transfer_request, body, self.excluded_keys)

        # Test RBAC
        # Note: The create service client does not define a target project
        #       ID, so everyone should be able to see it.
        expected_allowed = ['os_admin', 'os_primary', 'os_alt',
                            'os_project_member', 'os_project_reader']

        self.check_list_show_RBAC_enforcement(
            'TransferRequestClient', 'show_transfer_request', expected_allowed,
            True, transfer_request['id'])

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'TransferRequestClient', 'show_transfer_request', expected_allowed,
            True, transfer_request['id'], headers=self.all_projects_header)
        # TODO(johnsom) Move this down to the impersonate test below when the
        #               bug is resolved and the test is not skipped.
        self.check_list_show_RBAC_enforcement(
            'TransferRequestClient', 'show_transfer_request', expected_allowed,
            True, transfer_request['id'],
            headers={'x-auth-sudo-project-id': self.client.project_id})

    @decorators.idempotent_id('5bed4582-9cfb-11eb-a160-74e5f9e2a801')
    @decorators.skip_because(bug="1926572")
    def test_show_transfer_request_impersonate_another_project(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="show_transfer_request_impersonate", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone transfer_request')
        transfer_request = self.client.create_transfer_request(zone['id'])[1]
        self.addCleanup(self.client.delete_transfer_request,
                        transfer_request['id'])

        LOG.info('As Admin tenant fetch the transfer_request without using '
                 '"x-auth-sudo-project-id" HTTP header. Expected: 404')
        self.assertRaises(lib_exc.NotFound,
                          lambda: self.admin_client.show_transfer_request(
                              transfer_request['id']))

        LOG.info('As Admin tenant fetch the transfer_request using '
                 '"x-auth-sudo-project-id" HTTP header.')
        body = self.admin_client.show_transfer_request(
            transfer_request['id'],
            headers={'x-auth-sudo-project-id': zone['project_id']})[1]

        LOG.info('Ensure the fetched response matches the '
                 'created transfer_request')
        self.assertExpected(transfer_request, body, self.excluded_keys)

    @decorators.idempotent_id('235ded87-0c47-430b-8cad-4f3194b927a6')
    def test_show_transfer_request_as_target(self):
        # Checks the target of a scoped transfer request can see
        # the request.
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_transfer_request_as_target", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        transfer_request_data = dns_data_utils.rand_transfer_request_data(
            target_project_id=self.os_alt.credentials.project_id)

        LOG.info('Create a scoped zone transfer_request')
        transfer_request = self.client.create_transfer_request(
            zone['id'], transfer_request_data)[1]
        self.addCleanup(self.client.delete_transfer_request,
                        transfer_request['id'])

        LOG.info('Fetch the transfer_request as the target')
        body = self.alt_client.show_transfer_request(transfer_request['id'])[1]

        LOG.info('Ensure the fetched response matches the '
                 'created transfer_request')
        excluded_keys = self.excluded_keys + ["target_project_id",
                                              "project_id"]
        self.assertExpected(transfer_request, body, excluded_keys)

        # Test RBAC when a transfer target project is specified.
        if CONF.enforce_scope.designate:
            expected_allowed = ['os_primary', 'os_alt',
                                'os_project_member']
        else:
            expected_allowed = ['os_primary', 'os_alt', 'os_admin',
                                'os_project_member']

        self.check_list_show_RBAC_enforcement(
            'TransferRequestClient', 'show_transfer_request', expected_allowed,
            True, transfer_request['id'])

    @decorators.idempotent_id('7d81c487-aa15-44c4-b3e5-424ab9e6a3e5')
    def test_delete_transfer_request(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="delete_transfer_request", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a transfer_request')
        transfer_request = self.client.create_transfer_request(zone['id'])[1]
        self.addCleanup(self.client.delete_transfer_request,
                        transfer_request['id'],
                        ignore_errors=lib_exc.NotFound)

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')

        self.check_CUD_RBAC_enforcement(
            'TransferRequestClient', 'delete_transfer_request',
            expected_allowed, True, transfer_request['id'])

        LOG.info('Delete the transfer_request')
        self.client.delete_transfer_request(transfer_request['id'])
        self.assertRaises(lib_exc.NotFound,
            lambda: self.client.show_transfer_request(transfer_request['id']))

    @decorators.idempotent_id('ddd42a19-1768-428c-846e-32f9d6493011')
    def test_list_transfer_requests(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_transfer_request", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone transfer_request')
        transfer_request = self.client.create_transfer_request(zone['id'])[1]
        self.addCleanup(self.client.delete_transfer_request,
                        transfer_request['id'])

        LOG.info('List transfer_requests')
        body = self.client.list_transfer_requests()[1]

        self.assertGreater(len(body['transfer_requests']), 0)

        # Test RBAC - Users that are allowed to call list, but should get
        #             zero zones.
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed = ['os_admin']
        else:
            expected_allowed = ['os_alt']

        self.check_list_RBAC_enforcement_count(
            'TransferRequestClient', 'list_transfer_requests',
            expected_allowed, 0)

        # Test that users who should see the zone, can see it.
        expected_allowed = ['os_primary']

        self.check_list_IDs_RBAC_enforcement(
            'TransferRequestClient', 'list_transfer_requests',
            expected_allowed, [transfer_request['id']])

    @decorators.idempotent_id('db985892-9d02-11eb-a160-74e5f9e2a801')
    def test_list_transfer_requests_all_projects(self):
        LOG.info('Create a Primary zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_transfer_request_all_projects", suffix=self.tld_name)
        primary_zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete,
                        self.zones_client, primary_zone['id'])

        LOG.info('Create an Alt zone')
        alt_zone_name = dns_data_utils.rand_zone_name(
            name="list_transfer_request_all_projects_alt",
            suffix=self.tld_name)
        alt_zone = self.alt_zone_client.create_zone(name=alt_zone_name)[1]
        self.addCleanup(self.wait_zone_delete,
                        self.alt_zone_client, alt_zone['id'])

        LOG.info('Create a zone transfer_request using Primary client')
        primary_transfer_request = self.client.create_transfer_request(
            primary_zone['id'])[1]
        self.addCleanup(self.client.delete_transfer_request,
                        primary_transfer_request['id'])

        LOG.info('Create a zone transfer_request using Alt client')
        alt_transfer_request = self.alt_client.create_transfer_request(
            alt_zone['id'])[1]
        self.addCleanup(self.alt_client.delete_transfer_request,
                        alt_transfer_request['id'])

        LOG.info('List transfer_requests for all projects using Admin tenant '
                 'without "x-auth-all-projects" HTTP header. '
                 'Expected: empty list')
        self.assertEqual([], self.admin_client.list_transfer_requests()[1][
            'transfer_requests'], 'Failed, requests list is not empty')

        LOG.info('List transfer_requests for all projects using Admin tenant '
                 'and "x-auth-all-projects" HTTP header.')
        # Note: This is an all-projects list call, so other tests running
        #       in parallel will impact the list result set. Since the default
        #       pagination limit is only 20, we set a param limit of 1000 here.
        request_ids = [
            item['id'] for item in self.admin_client.list_transfer_requests(
                headers=self.all_projects_header,
                params={'limit': 1000})[1]['transfer_requests']]

        for request_id in [primary_transfer_request['id'],
                           alt_transfer_request['id']]:
            self.assertIn(request_id, request_ids,
                          "Failed, transfer request ID:{} wasn't found in "
                          "listed IDs{}".format(request_id, request_ids))

        # Test RBAC with x-auth-all-projects
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'TransferRequestClient', 'list_transfer_requests',
            expected_allowed, [primary_transfer_request['id']],
            headers=self.all_projects_header)

    @decorators.idempotent_id('bee42f38-e666-4b85-a710-01f40ea1e56a')
    def test_list_transfer_requests_impersonate_another_project(self):
        LOG.info('Create a Primary zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_transfer_request_impersonate", suffix=self.tld_name)
        primary_zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete,
                        self.zones_client, primary_zone['id'])

        LOG.info('Create an Alt zone')
        alt_zone_name = dns_data_utils.rand_zone_name(
            name="list_transfer_request_impersonate_alt", suffix=self.tld_name)
        alt_zone = self.alt_zone_client.create_zone(name=alt_zone_name)[1]
        self.addCleanup(self.wait_zone_delete,
                        self.alt_zone_client, alt_zone['id'])

        LOG.info('Create a zone transfer_request using Primary client')
        primary_transfer_request = self.client.create_transfer_request(
            primary_zone['id'])[1]
        self.addCleanup(self.client.delete_transfer_request,
                        primary_transfer_request['id'])

        LOG.info('Create a zone transfer_request using Alt client')
        alt_transfer_request = self.alt_client.create_transfer_request(
            alt_zone['id'])[1]
        self.addCleanup(self.alt_client.delete_transfer_request,
                        alt_transfer_request['id'])

        request_ids = [
            item['id'] for item in self.admin_client.list_transfer_requests(
                headers={'x-auth-sudo-project-id': self.alt_client.project_id},
                params={'limit': 1000})[1]['transfer_requests']]

        self.assertEqual([alt_transfer_request['id']], request_ids)

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'TransferRequestClient', 'list_transfer_requests',
            expected_allowed, [primary_transfer_request['id']],
            headers={'x-auth-sudo-project-id': self.client.project_id})

    @decorators.idempotent_id('de5e9d32-c723-4518-84e5-58da9722cc13')
    def test_update_transfer_request(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="update_transfer_request", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone transfer_request')
        transfer_request = self.client.create_transfer_request(zone['id'])[1]
        self.addCleanup(self.client.delete_transfer_request,
                        transfer_request['id'])

        LOG.info('Update the transfer_request')
        data = {
                 "description": "demo description"
               }
        transfer_request_patch = self.client.update_transfer_request(
            transfer_request['id'], transfer_request_data=data)[1]

        self.assertEqual(data['description'],
                         transfer_request_patch['description'])

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement(
            'TransferRequestClient', 'update_transfer_request',
            expected_allowed, True,
            transfer_request['id'], transfer_request_data=data)

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin', 'os_primary']

        self.check_CUD_RBAC_enforcement(
            'TransferRequestClient', 'update_transfer_request',
            expected_allowed, False,
            transfer_request['id'], transfer_request_data=data,
            headers=self.all_projects_header)
        self.check_CUD_RBAC_enforcement(
            'TransferRequestClient', 'update_transfer_request',
            expected_allowed, False,
            transfer_request['id'], transfer_request_data=data,
            headers={'x-auth-sudo-project-id': self.client.project_id})

    @decorators.idempotent_id('73b754a9-e856-4fd6-80ba-e8d1b80f5dfa')
    def test_list_transfer_requests_dot_json_fails(self):
        uri = self.client.get_uri('transfer_requests.json')

        self.assertRaises(lib_exc.NotFound,
            lambda: self.client.get(uri))


class TestTransferRequestNotFound(BaseTransferRequestTest):
    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestTransferRequestNotFound, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestTransferRequestNotFound, cls).setup_clients()
        cls.client = cls.os_primary.dns_v2.TransferRequestClient()

    @decorators.idempotent_id('39131f7c-e9bb-4f92-a325-444a675e1b3d')
    def test_create_transfer_request_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.client.create_transfer_request,
                              data_utils.rand_uuid())
        self.assertEqual(404, e.resp.status)
        self.assertEqual(404, e.resp_body['code'])
        self.assertEqual("zone_not_found", e.resp_body['type'])

    @decorators.idempotent_id('d255f72f-ba24-43df-9dba-011ed7f4625d')
    def test_show_transfer_request_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.client.show_transfer_request,
                              data_utils.rand_uuid())
        self.assertTransferRequest404(e.resp, e.resp_body)

    @decorators.idempotent_id('9ff383fb-c31d-4c6f-8085-7b261e401223')
    def test_update_transfer_request_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.client.update_transfer_request,
                              data_utils.rand_uuid())
        self.assertTransferRequest404(e.resp, e.resp_body)

    @decorators.idempotent_id('5a4a0755-c01d-448f-b856-b081b96ae77e')
    def test_delete_transfer_request_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.client.delete_transfer_request,
                              data_utils.rand_uuid())
        self.assertTransferRequest404(e.resp, e.resp_body)

    def assertTransferRequest404(self, resp, resp_body):
        self.assertEqual(404, resp.status)
        self.assertEqual(404, resp_body['code'])
        self.assertEqual("zone_transfer_request_not_found", resp_body['type'])


class TestTransferRequestInvalidId(BaseTransferRequestTest):
    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestTransferRequestInvalidId, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestTransferRequestInvalidId, cls).setup_clients()
        cls.client = cls.os_primary.dns_v2.TransferRequestClient()

    @decorators.idempotent_id('2205dd19-ecc7-4c68-9e89-63c47d642b07')
    def test_show_transfer_request_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.client.show_transfer_request,
                              'foo')
        self.assertTransferRequestInvalidId(e.resp, e.resp_body)

    @decorators.idempotent_id('af0ce46f-10be-4cce-a1d5-1b5c2a39fb97')
    def test_update_transfer_request_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.client.update_transfer_request,
                              'foo')
        self.assertTransferRequestInvalidId(e.resp, e.resp_body)

    @decorators.idempotent_id('1728dca5-01f1-45f4-b59d-7a981d479394')
    def test_delete_transfer_request_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.client.delete_transfer_request,
                              'foo')
        self.assertTransferRequestInvalidId(e.resp, e.resp_body)

    def assertTransferRequestInvalidId(self, resp, resp_body):
        self.assertEqual(400, resp.status)
        self.assertEqual(400, resp_body['code'])
        self.assertEqual("invalid_uuid", resp_body['type'])



# Copyright 2016 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import os

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators

from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin.common import waiters
from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.tests.api.v2.test_zones_exports import (
    BaseZoneExportsTest)

CONF = config.CONF
LOG = logging.getLogger(__name__)


class ZonesExportTest(BaseZoneExportsTest):
    credentials = ["primary", "admin"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZonesExportTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ZonesExportTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.ZoneExportsClient()
        cls.client = cls.os_primary.dns_v2.ZoneExportsClient()
        cls.recordset_client = cls.os_primary.dns_v2.RecordsetClient()

    def _create_zone_export(self, test_name):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name=test_name, suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone export')
        zone_export = self.client.create_zone_export(
            zone['id'], wait_until=const.COMPLETE)[1]
        self.addCleanup(self.client.delete_zone_export, zone_export['id'])

        return zone, zone_export

    @decorators.idempotent_id('0484c3c4-df57-458e-a6e5-6eb63e0475e0')
    def test_create_zone_export_and_show_exported_zonefile(self):
        zone, zone_export = self._create_zone_export(
            'create_zone_export_and_show_exported_zonefile')

        self.assertEqual(const.PENDING, zone_export['status'])
        self.assertEqual(zone['id'], zone_export['zone_id'])
        self.assertIsNone(zone_export['links'].get('export'))
        self.assertIsNone(zone_export['location'])

        LOG.info('Check the zone export looks good')
        _, zone_export = self.client.show_zone_export(zone_export['id'])

        self.assertEqual(const.COMPLETE, zone_export['status'])
        self.assertEqual(zone['id'], zone_export['zone_id'])
        self.assertIsNotNone(zone_export['links'].get('export'))
        self.assertIsNotNone(zone_export['location'])

        LOG.info('Fetch the exported zonefile')
        _, zonefile = self.client.show_exported_zonefile(zone_export['id'])
        self.assertEqual(zone['name'], zonefile.origin)
        self.assertEqual(zone['ttl'], zonefile.ttl)

    @decorators.idempotent_id('56b8f30e-cd45-4c7a-bc0c-bbf92d7dc697')
    def test_show_exported_zonefile_impersonate_another_project(self):
        zone, zone_export = self._create_zone_export(
            'show_exported_zonefile_impersonate')

        LOG.info('As Admin impersonate "primary" client,'
                 ' to show exported zone file')
        response = self.admin_client.show_exported_zonefile(
            zone_export['id'], headers={
                'x-auth-sudo-project-id': zone['project_id']})[1]
        self.assertEqual(zone['name'], response.origin)
        self.assertEqual(zone['ttl'], response.ttl)

    @decorators.idempotent_id('c2e55514-ff2e-41d9-a3cc-9e78873254c9')
    def test_show_exported_zonefile_all_projects(self):
        zone, zone_export = self._create_zone_export(
            'show_exported_zonefile_all_projects')
        resp_headers, resp_data = self.admin_client.show_exported_zonefile(
            zone_export['id'], headers={
                'x-auth-all-projects': True
            })
        self.assertEqual(zone['name'], resp_data.origin)
        self.assertEqual(zone['ttl'], resp_data.ttl)

    @decorators.idempotent_id('9746b7f2-2df4-448c-8a85-5ab6bf74f1fe')
    def test_show_exported_zonefile_any_mime_type(self):
        zone, zone_export = self._create_zone_export(
            'show_exported_zonefile_any_mime_type')
        resp_headers, resp_data = self.client.show_exported_zonefile(
            zone_export['id'], headers={'Accept': '*/*'})

        LOG.info('Ensure Content-Type: text/dns')
        self.assertIn(
            'text/dns', resp_headers['content-type'],
            "Failed, the expected 'Content-type:text/dns wasn't received.")

        LOG.info('Ensure exported data ia as expected')
        self.assertEqual(zone['name'], resp_data.origin)
        self.assertEqual(zone['ttl'], resp_data.ttl)

    @decorators.idempotent_id('dc7a9dde-d287-4e22-9788-26578f0d3bf0')
    def test_missing_accept_headers(self):
        zone, zone_export = self._create_zone_export(
            'missing_accept_headers')
        resp_headers, resp_data = self.client.show_exported_zonefile(
            zone_export['id'], headers={})
        LOG.info('Ensure Content-Type: text/dns')
        self.assertIn(
            'text/dns', resp_headers['content-type'],
            "Failed, the expected 'Content-type:text/dns wasn't received.")

        LOG.info('Ensure exported data ia as expected')
        self.assertEqual(zone['name'], resp_data.origin)
        self.assertEqual(zone['ttl'], resp_data.ttl)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('d8f444aa-a645-4a03-b366-46836f57dc69')
    def test_all_recordset_types_exist_in_show_zonefile(self):
        recorsets_data_file = os.path.join(
            os.path.dirname(__file__), 'recordset_data.json')

        if not os.path.exists(recorsets_data_file):
            raise self.skipException(
                f"Could not find {recorsets_data_file}")

        file = open(recorsets_data_file, "r")
        load_file = json.loads(file.read())
        file.close()

        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="all_recordset_types_exist", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                             wait_until=const.ACTIVE)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        created_records = []
        for record_data in load_file.values():
            recordset_data = {
                'name': f"{record_data['name']}.{zone['name']}",
                'type': record_data['type'],
                'records': record_data['records'],
            }
            try:
                LOG.info('Create a Recordset')
                recordset = self.recordset_client.create_recordset(
                    zone['id'], recordset_data)[1]
                self.addCleanup(self.wait_recordset_delete,
                                self.recordset_client, zone['id'],
                                recordset['id'])
                created_records.append(recordset['records'])
                waiters.wait_for_recordset_status(self.recordset_client,
                                                  zone['id'], recordset['id'],
                                                  const.ACTIVE)
            except Exception as err:
                LOG.warning(f"Record of type {recordset['type']} could not be"
                            f" created and failed with error: {err}")

        LOG.info('Create a zone export')
        zone_export = self.client.create_zone_export(
            zone['id'], wait_until=const.COMPLETE)[1]
        self.addCleanup(self.client.delete_zone_export, zone_export['id'])

        LOG.info('Show exported zonefile')
        created_zonefile = self.client.show_exported_zonefile(
            zone_export['id'])[1]

        file_records = [item.data for item in created_zonefile.records]
        for record in created_records:
            for r in record:
                self.assertIn(r, file_records,
                            f"Failed, missing record: {r} in zone file")



# Copyright 2016 Hewlett Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time
import math

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools

from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.tests import base
from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin.common import waiters
from designate_tempest_plugin.services.dns.query.query_client import (
    SingleQueryClient)

CONF = config.CONF

CONF = config.CONF
LOG = logging.getLogger(__name__)


class ZonesTest(base.BaseDnsV2Test):
    credentials = ["primary", "admin"]

    @classmethod
    def setup_clients(cls):
        super(ZonesTest, cls).setup_clients()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.rec_client = cls.os_admin.dns_v2.RecordsetClient()
        cls.primary_client = cls.os_primary.dns_v2.BlacklistsClient()

    @classmethod
    def resource_setup(cls):
        super(ZonesTest, cls).resource_setup()
        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="ZonesTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(ZonesTest, cls).resource_cleanup()

    @decorators.attr(type='smoke')
    @decorators.attr(type='slow')
    @decorators.idempotent_id('d0648f53-4114-45bd-8792-462a82f69d32')
    def test_create_and_delete_zone(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_and_delete_zone", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)

        LOG.info('Ensure we respond with CREATE+PENDING')
        self.assertEqual(const.CREATE, zone['action'])
        self.assertEqual(const.PENDING, zone['status'])

        waiters.wait_for_zone_status(
            self.zones_client, zone['id'], const.ACTIVE)

        LOG.info('Re-Fetch the zone')
        zone = self.zones_client.show_zone(zone['id'])[1]

        LOG.info('Ensure we respond with NONE+ACTIVE')
        self.assertEqual(const.NONE, zone['action'])
        self.assertEqual(const.ACTIVE, zone['status'])

        LOG.info('Delete the zone')
        zone = self.zones_client.delete_zone(zone['id'])[1]

        LOG.info('Ensure we respond with DELETE+PENDING')
        self.assertEqual(const.DELETE, zone['action'])
        self.assertEqual(const.PENDING, zone['status'])

        waiters.wait_for_zone_404(self.zones_client, zone['id'])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('cabd6334-ba37-11ec-9d8c-201e8823901f')
    def test_create_and_update_zone(self):

        LOG.info('Create a zone and wait until it becomes ACTIVE')
        orig_ttl = 666
        orig_description = 'test_create_and_update_zone: org description'
        zone_name = dns_data_utils.rand_zone_name(
            name="create_and_update_zone", suffix=self.tld_name)
        zone = self.zones_client.create_zone(
            name=zone_name,
            ttl=orig_ttl, description=orig_description,
            wait_until=const.ACTIVE)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)

        LOG.info("Update zone's: TTL and Description, wait until ACTIVE")
        updated_ttl = 777
        updated_description = dns_data_utils.rand_string(20)
        self.zones_client.update_zone(
            zone['id'], ttl=updated_ttl, description=updated_description,
            wait_until=const.ACTIVE)

        LOG.info('Re-Fetch/Show the zone')
        show_zone = self.zones_client.show_zone(zone['id'])[1]

        LOG.info('Ensure that the Description and TLL has been updated')
        self.assertEqual(
            updated_ttl, show_zone['ttl'],
            'Failed, actual TTL value:{} is not as expected:{} after '
            'the update)'.format(show_zone['ttl'], updated_ttl))
        self.assertEqual(
            updated_description, show_zone['description'],
            'Failed, actual Description:{} is not as expected:{} after '
            'the update)'.format(show_zone['description'], orig_description))

    @decorators.attr(type='slow')
    @decorators.idempotent_id('c9838adf-14dc-4097-9130-e5cea3727abb')
    def test_delete_zone_pending_create(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="delete_zone_pending_create", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)

        # NOTE(kiall): This is certainly a little racey, it's entirely
        #              possible the zone will become active before we delete
        #              it. Worst case, that means we get an unexpected pass.
        #              Theres not a huge amount we can do, given this is
        #              black-box testing.
        LOG.info('Delete the zone while it is still pending')
        zone = self.zones_client.delete_zone(zone['id'])[1]

        LOG.info('Ensure we respond with DELETE+PENDING')
        self.assertEqual(const.DELETE, zone['action'])
        self.assertEqual(const.PENDING, zone['status'])

        waiters.wait_for_zone_404(self.zones_client, zone['id'])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('ad8d1f5b-da66-46a0-bbee-14dc84a5d791')
    @testtools.skipUnless(
        config.CONF.dns.nameservers,
        "Config option dns.nameservers is missing or empty")
    def test_zone_create_propagates_to_nameservers(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="zone_create_propagates", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        waiters.wait_for_zone_status(self.zones_client, zone['id'],
                                     const.ACTIVE)
        waiters.wait_for_query(self.query_client, zone['name'], const.SOA)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('d13d3095-c78f-4aae-8fe3-a74ccc335c84')
    @testtools.skipUnless(
        config.CONF.dns.nameservers,
        "Config option dns.nameservers is missing or empty")
    def test_zone_delete_propagates_to_nameservers(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="zone_delete_propagates", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)

        waiters.wait_for_zone_status(self.zones_client, zone['id'],
                                     const.ACTIVE)
        waiters.wait_for_query(self.query_client, zone['name'], const.SOA)

        LOG.info('Delete the zone')
        self.zones_client.delete_zone(zone['id'])

        waiters.wait_for_zone_404(self.zones_client, zone['id'])
        waiters.wait_for_query(self.query_client, zone['name'], const.SOA,
                               found=False)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('ff9b9fc4-85b4-11ec-bcf5-201e8823901f')
    @testtools.skipUnless(
        config.CONF.dns.nameservers,
        "Config option dns.nameservers is missing or empty")
    def test_notify_msg_sent_to_nameservers(self):

        # Test will only run when the SOA record Refresh is close to one hour,
        # otherwise skipped.
        # This implies that the only reason "A" record was propagated is as a
        # result of successfully sent NOTIFY message.

        LOG.info('Create a zone, wait until ACTIVE and get the Serial'
                 ' and SOA Refresh values')
        zone_name = dns_data_utils.rand_zone_name(
            name="test_notify_msg_sent_to_nameservers", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                             wait_until='ACTIVE')[1]

        org_serial = zone['serial']
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        try:
            soa = [
                rec['records'] for rec in self.rec_client.list_recordset(
                    zone['id'], headers=self.all_projects_header)[1][
                    'recordsets'] if rec['type'] == 'SOA'][0]
            refresh = int(soa[0].split(' ')[3])
            if math.isclose(3600, refresh, rel_tol=0.1) is False:
                raise self.skipException(
                    'Test is skipped, actual SOA REFRESH is:{} unlike test'
                    ' prerequisites that requires a value close to 3600'
                    ' (one hour)'.format(refresh))
        except Exception as e:
            raise self.skipException(
                'Test is skipped, something went wrong on getting SOA REFRESH'
                ' value, the error was:{}'.format(e))

        LOG.info("Update Zone's TTL, wait until ACTIVE and"
                 " ensure Zone's Serial has changed")
        self.zones_client.update_zone(
            zone['id'], ttl=dns_data_utils.rand_ttl(), wait_until='ACTIVE')
        new_serial = self.zones_client.show_zone(zone['id'])[1]['serial']
        self.assertNotEqual(
            new_serial, org_serial,
            "Failed, expected behaviour is that the Designate DNS changes the"
            " Serial after updating Zone's TTL value")
        waiters.wait_for_query(self.query_client, zone['name'], "SOA")

        LOG.info('Per Nameserver "dig" for a SOA record until either:'
                 ' updated Serial is detected or build timeout has reached')
        for ns in config.CONF.dns.nameservers:
            start = time.time()
            while True:
                ns_obj = SingleQueryClient(ns, config.CONF.dns.query_timeout)
                ns_soa_record = ns_obj.query(zone['name'], rdatatype='SOA')
                if str(new_serial) in str(ns_soa_record):
                    return
                if time.time() - start >= config.CONF.dns.build_timeout:
                    raise lib_exc.TimeoutException(
                        'Failed, expected Serial:{} for a Zone was not'
                        ' detected on Nameserver:{} within a timeout of:{}'
                        ' seconds.'.format(
                            new_serial, ns, config.CONF.dns.build_timeout))



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from socket import gaierror

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools

from designate_tempest_plugin.common import waiters
from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.tests import base

from designate_tempest_plugin.services.dns.query.query_client import (
    SingleQueryClient)

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseZonesTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'version', 'links',
                    'status', 'action']

    @classmethod
    def setup_clients(cls):
        super(BaseZonesTest, cls).setup_clients()

        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(BaseZonesTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="BaseZonesTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BaseZonesTest, cls).resource_cleanup()


class ZoneTasks(BaseZonesTest):
    credentials = ["primary", "alt", "admin",
                   "project_member", "project_reader"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZoneTasks, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ZoneTasks, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.ZonesClient()
        cls.alt_client = cls.os_alt.dns_v2.ZonesClient()

    @decorators.idempotent_id('287e2cd0-a0e7-11eb-b962-74e5f9e2a801')
    @testtools.skipUnless(
        config.CONF.dns.nameservers,
        "Config option dns.nameservers is missing or empty")
    def test_zone_abandon(self):
        LOG.info('Create a PRIMARY zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="zone_abandon", suffix=self.tld_name)
        pr_zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client,
                        pr_zone['id'])
        waiters.wait_for_zone_status(self.zones_client, pr_zone['id'],
                                     'ACTIVE')

        LOG.info('Ensure we respond with CREATE+PENDING')
        self.assertEqual('CREATE', pr_zone['action'])
        self.assertEqual('PENDING', pr_zone['status'])

        LOG.info('Fetch the zone')
        self.zones_client.show_zone(pr_zone['id'])

        LOG.info('Check that the zone was created on Nameserver/BIND')
        waiters.wait_for_query(self.query_client, pr_zone['name'], "SOA")

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'ZonesClient', 'abandon_zone', expected_allowed, False,
            pr_zone['id'],
            headers={'x-auth-sudo-project-id': pr_zone['project_id']})

        # Test abandoning the zone
        LOG.info('Abandon a zone')
        self.admin_client.abandon_zone(
            pr_zone['id'],
            headers={'x-auth-sudo-project-id': pr_zone['project_id']})

        LOG.info('Wait for the zone to become 404/NotFound in Designate')
        waiters.wait_for_zone_404(self.zones_client, pr_zone['id'])

        LOG.info('Check that the zone is still exists in Nameserver/BIND')
        waiters.wait_for_query(
            self.query_client, pr_zone['name'], "SOA")

    @decorators.idempotent_id('90b21d1a-a1ba-11eb-84fa-74e5f9e2a801')
    @testtools.skipUnless(
        config.CONF.dns.nameservers,
        "Config option dns.nameservers is missing or empty")
    def test_zone_abandon_forbidden(self):

        LOG.info('Create a PRIMARY zone and add to the cleanup')
        zone_name = dns_data_utils.rand_zone_name(
            name="zone_abandon_forbidden", suffix=self.tld_name)
        pr_zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client,
                        pr_zone['id'])
        waiters.wait_for_zone_status(self.zones_client, pr_zone['id'],
                                     'ACTIVE')

        LOG.info('Ensure we respond with CREATE+PENDING')
        self.assertEqual('CREATE', pr_zone['action'])
        self.assertEqual('PENDING', pr_zone['status'])

        LOG.info('Fetch the zone')
        self.zones_client.show_zone(pr_zone['id'])

        LOG.info('Check that the zone was created on Nameserver/BIND')
        waiters.wait_for_query(self.query_client, pr_zone['name'], "SOA")

        LOG.info('Abandon a zone as primary client, Expected: should '
                 'fail with: 403 forbidden')
        self.assertRaises(
            lib_exc.Forbidden, self.zones_client.abandon_zone,
            zone_id=pr_zone['id'])


class ZoneTasksNegative(BaseZonesTest):
    credentials = ["primary", "alt", "admin"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZoneTasksNegative, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ZoneTasksNegative, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.ZonesClient()
        cls.alt_client = cls.os_alt.dns_v2.ZonesClient()

    def _query_nameserver(self, nameserver, query_timeout,
                          zone_name, zone_type='SOA'):
        query_succeeded = False
        ns_obj = SingleQueryClient(nameserver, query_timeout)
        try:
            ns_obj.query(zone_name, zone_type)
            query_succeeded = True
        except gaierror as e:
            LOG.info('Function "_query_nameserver" failed with:{} '.format(e))
        return query_succeeded



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import random
from oslo_log import log as logging
from oslo_utils import versionutils
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.lib.common.utils import data_utils

import tempest.test

from designate_tempest_plugin.tests import base
from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin.common import exceptions


LOG = logging.getLogger(__name__)


CONF = config.CONF


class QuotasV2Test(base.BaseDnsV2Test):

    credentials = ['primary', 'admin', 'alt']
    test_quota_limit = 3

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(QuotasV2Test, cls).setup_credentials()

    @classmethod
    def skip_checks(cls):
        super(QuotasV2Test, cls).skip_checks()

        if not CONF.dns_feature_enabled.api_v2_quotas:
            skip_msg = ("%s skipped as designate V2 Quotas API is not "
                        "available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(QuotasV2Test, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.QuotasClient()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.quotas_client = cls.os_primary.dns_v2.QuotasClient()
        cls.alt_client = cls.os_alt.dns_v2.QuotasClient()
        cls.alt_zone_client = cls.os_alt.dns_v2.ZonesClient()
        cls.recordset_client = cls.os_primary.dns_v2.RecordsetClient()

    @classmethod
    def resource_setup(cls):
        super(QuotasV2Test, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="QuotasV2Test")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(QuotasV2Test, cls).resource_cleanup()

    def _set_quota_for_project(self, project_id, quotas):
        http_header = {'x-auth-sudo-project-id': project_id}
        self.admin_client.set_quotas(
            project_id=project_id, quotas=quotas, headers=http_header)
        self.addCleanup(
            self.admin_client.delete_quotas,
            project_id=project_id, headers=http_header)

    def _reach_quota_limit(
            self, limit_threshold, quota_type, zone=None):
        attempt_number = 0
        not_raised_msg = ("Failed, expected '413 over_quota' response of "
                          "type:{} wasn't received.".format(quota_type))
        while attempt_number <= limit_threshold + 1:
            try:
                attempt_number += 1
                LOG.info('Attempt No:{} '.format(attempt_number))
                if quota_type == 'zones_quota':
                    zone_name = dns_data_utils.rand_zone_name(
                        name="_reach_quota_limit", suffix=self.tld_name)
                    zone = self.zones_client.create_zone(
                        name=zone_name,
                        description='Test zone for:{}'.format(quota_type))[1]
                    self.addCleanup(
                        self.wait_zone_delete,
                        self.zones_client, zone['id'])
                else:
                    if quota_type == 'zone_recordsets':
                        max_number_of_records = 10
                        prj_quota = self.admin_client.show_quotas(
                            project_id=self.zones_client.project_id,
                            headers=self.all_projects_header)[1][
                            'zone_records']
                        if max_number_of_records > prj_quota:
                            max_number_of_records = prj_quota
                        recordset_data = dns_data_utils.rand_recordset_data(
                            record_type='A', zone_name=zone['name'],
                            number_of_records=random.randint(
                                1, max_number_of_records))
                    else:
                        recordset_data = dns_data_utils.rand_recordset_data(
                            record_type='A', zone_name=zone['name'])
                    recordset = self.recordset_client.create_recordset(
                        zone['id'], recordset_data=recordset_data,
                        wait_until=const.ACTIVE)[1]
                    self.addCleanup(
                        self.wait_recordset_delete,
                        self.recordset_client,
                        zone['id'], recordset['id'])
                self.assertLess(
                    attempt_number, limit_threshold + 1, not_raised_msg)
            except Exception as e:
                raised_err = str(e).replace(' ', '')
                if not_raised_msg in str(e):
                    raise AssertionError(not_raised_msg)
                elif ("'code':413" in raised_err and
                      "'type':'over_quota'" in raised_err):
                    LOG.info("OK, type':'over_quota' was raised")
                    break
                else:
                    raise

    @decorators.attr(type='slow')
    @decorators.idempotent_id('41d9cf2c-866a-11ec-8ccb-201e8823901f')
    @decorators.skip_because(bug="1960495")
    def test_api_export_size_quota(self):
        LOG.info('Admin sets "api_export_size:{}" quota for Primary'
                 ' user'.format(self.test_quota_limit))
        quotas = dns_data_utils.rand_quotas()
        quotas['api_export_size'] = self.test_quota_limit
        self._set_quota_for_project(
            self.zones_client.project_id, quotas)
        LOG.info('Create a Zone, wait until ACTIVE and add:{}'
                 ' Recordsets'.format(self.test_quota_limit + 1))
        zone = self.zones_client.create_zone(
            description='Zone for test_api_export_size_quota',
            wait_until=const.ACTIVE)[1]
        self.addCleanup(
            self.wait_zone_delete,
            self.zones_client, zone['id'])
        for i in range(self.test_quota_limit + 1):
            recordset_data = dns_data_utils.rand_recordset_data(
                record_type='A', zone_name=zone['name'])
            LOG.info('Try to create a recordset No:{}'.format(i))
            recordset = self.recordset_client.create_recordset(
                zone['id'], recordset_data=recordset_data,
                wait_until=const.ACTIVE)[1]
            self.addCleanup(
                self.wait_recordset_delete,
                self.recordset_client,
                zone['id'], recordset['id'])
        LOG.info(
            'Ensure that the Number of Recordsets is bigger than configured'
            ' api_export_size:{}'.format(self.test_quota_limit))
        number_of_recordsets = len(self.recordset_client.list_recordset(
            zone['id'])[1]['recordsets'])
        self.assertGreater(
            number_of_recordsets, self.test_quota_limit,
            'Failed, the number of recordsets within a Zone is not enough to'
            ' trigger "413 over quota" on Zone Export')
        LOG.info('Try to export Zone. Expected:"413 over_quota"')
        with self.assertRaisesDns(
                lib_exc.OverLimit, 'over_quota', 413):
            self.export_zone_client.create_zone_export(zone['id'])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('2513cb6e-85ec-11ec-bf7f-201e8823901f')
    def test_recordset_records_quota(self):
        LOG.info('Admin sets "recordset_records:{}" quota for Primary'
                 ' user'.format(self.test_quota_limit))
        quotas = dns_data_utils.rand_quotas()
        quotas['recordset_records'] = self.test_quota_limit
        self._set_quota_for_project(
            self.zones_client.project_id, quotas)
        LOG.info('Create a Zone and wait until ACTIVE')
        zone_name = dns_data_utils.rand_zone_name(
            name="test_recordset_records_quota", suffix=self.tld_name)
        zone = self.zones_client.create_zone(
            name=zone_name,
            description='Zone for test_recordset_records_quota',
            wait_until=const.ACTIVE)[1]
        self.addCleanup(
            self.wait_zone_delete,
            self.zones_client, zone['id'])
        LOG.info(
            'Create recordset data with:{} records and try to create'
            ' a recordset. Expected:"413 over_quota"'.format(
                self.test_quota_limit + 1))
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=zone['name'],
            number_of_records=self.test_quota_limit + 1)
        LOG.info('Try to create a recordset. Expected:"413 over_quota"')
        with self.assertRaisesDns(
                lib_exc.OverLimit, 'over_quota', 413):
            self.recordset_client.create_recordset(
                zone['id'], recordset_data=recordset_data)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('893dc648-868d-11ec-8ccb-201e8823901f')
    def test_zone_records_quota(self):
        LOG.info('Create a Zone and wait until ACTIVE')
        zone_name = dns_data_utils.rand_zone_name(
            name="test_zone_records_quota", suffix=self.tld_name)
        zone = self.zones_client.create_zone(
            name=zone_name,
            description='Zone for test_zone_records_quota',
            wait_until=const.ACTIVE)[1]
        self.addCleanup(
            self.wait_zone_delete,
            self.zones_client, zone['id'])
        LOG.info('Admin sets "zone_records:{}" quota for Primary '
                 'user'.format(self.test_quota_limit))
        quotas = dns_data_utils.rand_quotas()
        quotas['zone_records'] = self.test_quota_limit
        self._set_quota_for_project(
            self.zones_client.project_id, quotas)
        LOG.info(
            'Try to add:{} recordsets (with a single record) to the Zone in'
            ' loop. Expected:"413 over_quota"'.format(
                self.test_quota_limit + 1))
        self._reach_quota_limit(
            self.test_quota_limit + 1, 'zone_records', zone)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('f567bdda-86b3-11ec-8ccb-201e8823901f')
    def test_zone_recordsets_quota(self):
        LOG.info('Create a Zone and wait until ACTIVE')
        zone_name = dns_data_utils.rand_zone_name(
            name="test_zone_recordsets_quota", suffix=self.tld_name)
        zone = self.zones_client.create_zone(
            name=zone_name,
            description='Zone for test_zone_recordsets_quota',
            wait_until=const.ACTIVE)[1]
        self.addCleanup(
            self.wait_zone_delete,
            self.zones_client, zone['id'])
        LOG.info('Admin sets "zone_recordsets:{}" quota for Primary '
                 'user'.format(self.test_quota_limit))
        quotas = dns_data_utils.rand_quotas()
        quotas['zone_recordsets'] = self.test_quota_limit
        self._set_quota_for_project(
            self.zones_client.project_id, quotas)
        LOG.info(
            'Try to add:{} recordsets (with a random number of records) to a'
            ' Zone in loop. Expected:"413 over_quota"'.format(
                self.test_quota_limit + 1))
        self._reach_quota_limit(
            self.test_quota_limit + 1,
            'zone_recordsets', zone)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('6987953a-dccf-11eb-903e-74e5f9e2a801')
    def test_zones_quota(self):
        LOG.info('Admin sets "zones" quota for Primary user')
        quotas = dns_data_utils.rand_quotas()
        quotas['zones'] = self.test_quota_limit
        self._set_quota_for_project(
            self.zones_client.project_id, quotas)
        LOG.info('Try to create Zones. Expected:"413 over_quota"')
        self._reach_quota_limit(self.test_quota_limit, 'zones_quota')


class QuotasBoundary(base.BaseDnsV2Test, tempest.test.BaseTestCase):

    credentials = ['admin', 'primary']

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(QuotasBoundary, cls).setup_credentials()

    @classmethod
    def skip_checks(cls):
        super(QuotasBoundary, cls).skip_checks()
        if not CONF.dns_feature_enabled.api_v2_quotas:
            skip_msg = ("%s skipped as designate V2 Quotas API is not "
                        "available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(QuotasBoundary, cls).setup_clients()
        cls.quota_client = cls.os_admin.dns_v2.QuotasClient()
        cls.project_client = cls.os_admin.projects_client
        cls.admin_zones_client = cls.os_admin.dns_v2.ZonesClient()
        cls.recordset_client = cls.os_admin.dns_v2.RecordsetClient()
        cls.export_zone_client = cls.os_admin.dns_v2.ZoneExportsClient()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(QuotasBoundary, cls).resource_setup()
        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="QuotasBoundary")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(QuotasBoundary, cls).resource_cleanup()

    @decorators.attr(type='slow')
    @decorators.idempotent_id('e4981eb2-3803-11ed-9d3c-201e8823901f')
    def test_zone_quota_boundary(self):
        # Create a dedicated Project for Boundary tests
        tenant_id = self.project_client.create_project(
            name=data_utils.rand_name(name='BoundaryZone'))['project']['id']
        self.addCleanup(self.project_client.delete_project, tenant_id)

        # Set Quotas (zones:1) for tested project
        sudo_header = {'x-auth-sudo-project-id': tenant_id}
        quotas = {
            'zones': 1, 'zone_recordsets': 2, 'zone_records': 3,
            'recordset_records': 2, 'api_export_size': 3}
        self.quota_client.set_quotas(
            project_id=tenant_id, quotas=quotas,
            headers=sudo_header)

        # Create a first Zone --> Should PASS
        zone_name = dns_data_utils.rand_zone_name(
            name="test_zone_quota_boundary_attempt_1", suffix=self.tld_name)
        zone = self.admin_zones_client.create_zone(
            name=zone_name, project_id=tenant_id)[1]
        self.addCleanup(self.wait_zone_delete, self.admin_zones_client,
                        zone['id'])

        # Create a second zone --> should FAIL on: 413 over_quota
        zone_name = dns_data_utils.rand_zone_name(
            name="test_zone_quota_boundary_attempt_2", suffix=self.tld_name)
        try:
            response_headers, zone = self.admin_zones_client.create_zone(
                name=zone_name, project_id=tenant_id)
            if response_headers['status'] != 413:
                raise exceptions.InvalidStatusError(
                    'Zone', zone['id'], zone['status'])
        except Exception as e:
            self.assertIn('over_quota', str(e),
                          'Failed, over_quota for a zone was not raised')
        finally:
            self.addCleanup(
                self.wait_zone_delete,
                self.admin_zones_client, zone['id'],
                headers=sudo_header,
                ignore_errors=lib_exc.NotFound)


class SharedZonesQuotaTest(base.BaseDnsV2Test):
    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(SharedZonesQuotaTest, cls).setup_clients()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.adm_project_client = cls.os_admin.projects_client
        cls.adm_quota_client = cls.os_admin.dns_v2.QuotasClient()
        cls.adm_zone_client = cls.os_admin.dns_v2.ZonesClient()
        cls.adm_shr_client = cls.os_admin.dns_v2.SharedZonesClient()
        cls.share_zone_client = cls.os_primary.dns_v2.SharedZonesClient()
        cls.rec_client = cls.os_primary.dns_v2.RecordsetClient()
        cls.export_zone_client = cls.os_primary.dns_v2.ZoneExportsClient()

    @classmethod
    def resource_setup(cls):
        super(SharedZonesQuotaTest, cls).resource_setup()

        if not versionutils.is_compatible('2.1', cls.api_version,
                                          same_major=False):
            raise cls.skipException(
                'The shared zones scenario tests require Designate API '
                'version 2.1 or newer. Skipping Shared Zones scenario tests.')

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name='SharedZonesTest')
        cls.tld_name = f'.{tld_name}'
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(SharedZonesQuotaTest, cls).resource_cleanup()

    def _create_shared_zone_for_project(
            self, zone_name, project_id, sudo_header):
        """Admin creates Zone for project ID and shares it with Primary"""
        zone_name = dns_data_utils.rand_zone_name(
            name=zone_name,
            suffix=self.tld_name)
        zone = self.adm_zone_client.create_zone(
            name=zone_name, project_id=project_id, wait_until=const.ACTIVE)[1]
        self.addCleanup(
            self.wait_zone_delete, self.adm_zone_client, zone['id'],
            headers=sudo_header, delete_shares=True)
        shared_zone = self.adm_shr_client.create_zone_share(
            zone['id'], self.rec_client.project_id,
            headers=sudo_header)[1]
        self.addCleanup(self.adm_shr_client.delete_zone_share,
                        zone['id'], shared_zone['id'], headers=sudo_header)
        return zone, shared_zone

    @decorators.attr(type='slow')
    @decorators.idempotent_id('31968688-c0d3-11ed-b04f-201e8823901f')
    @decorators.skip_because(bug="1992445")
    def test_zone_recordsets_enforced_against_owner(self):

        # Create a dedicated Project "A" for shared zone test
        tenant_id = self.adm_project_client.create_project(
            name=data_utils.rand_name(
                name='SharedZonesQuotaTest'))['project']['id']
        self.addCleanup(self.adm_project_client.delete_project, tenant_id)

        # Set Quotas "zone_recordsets:1" for a project "A"
        sudo_header = {'x-auth-sudo-project-id': tenant_id}
        quotas = {
            'zones': 7, 'zone_recordsets': 1, 'zone_records': 7,
            'recordset_records': 7, 'api_export_size': 7}
        self.adm_quota_client.set_quotas(
            project_id=tenant_id, quotas=quotas,
            headers=sudo_header)

        # Admin creates a zone for project "A" and shares it Primary
        zone = self._create_shared_zone_for_project(
            zone_name='test_zone_recordsets_enforced_against_owner',
            project_id=tenant_id, sudo_header=sudo_header)[0]

        # Primary creates a first recodset - should PASS
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=zone['name'])
        recordset = self.rec_client.create_recordset(
            zone['id'], recordset_data)[1]
        self.addCleanup(self.wait_recordset_delete, self.rec_client,
            zone['id'], recordset['id'], ignore_errors=lib_exc.NotFound)

        # Primary creates a second recodset - should FAIL
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=zone['name'])
        with self.assertRaisesDns(
                lib_exc.OverLimit, 'over_quota', 413):
            self.rec_client.create_recordset(zone['id'], recordset_data)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('cbb8d6b4-c64e-11ed-80d8-201e8823901f')
    def test_zone_recocrds_enforced_against_owner(self):

        # Create a dedicated Project "A" for shared zone test
        tenant_id = self.adm_project_client.create_project(
            name=data_utils.rand_name(
                name='SharedZonesQuotaTest'))['project']['id']
        self.addCleanup(self.adm_project_client.delete_project, tenant_id)

        # Set Quotas "zone_records:1" for a project "A"
        sudo_header = {'x-auth-sudo-project-id': tenant_id}
        quotas = {
            'zones': 7, 'zone_recordsets': 7, 'zone_records': 1,
            'recordset_records': 7, 'api_export_size': 7}
        self.adm_quota_client.set_quotas(
            project_id=tenant_id, quotas=quotas,
            headers=sudo_header)

        # Admin creates a zone for project "A" and share it with Primary
        zone = self._create_shared_zone_for_project(
            zone_name='test_zone_recocrds_enforced_against_owner',
            project_id=tenant_id, sudo_header=sudo_header)[0]

        # Primary creates recordset with (single record) --> Should PASS
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=zone['name'])
        body = self.rec_client.create_recordset(
            zone['id'], recordset_data)[1]
        self.addCleanup(
            self.wait_recordset_delete, self.rec_client,
            zone['id'], body['id'])

        # Primary creates one more recordset (single record) --> Should FAIL
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=zone['name'])
        with self.assertRaisesDns(
                lib_exc.OverLimit, 'over_quota', 413):
            self.rec_client.create_recordset(
                zone['id'], recordset_data)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('40b436f0-c895-11ed-8900-201e8823901f')
    def test_recordset_records_enforced_against_owner(self):

        # Create a dedicated Project "A" for shared zone test
        tenant_id = self.adm_project_client.create_project(
            name=data_utils.rand_name(
                name='SharedZonesQuotaTest'))['project']['id']
        self.addCleanup(self.adm_project_client.delete_project, tenant_id)

        # Set Quotas "zone_records:1" for a project "A"
        sudo_header = {'x-auth-sudo-project-id': tenant_id}
        quotas = {
            'zones': 7, 'zone_recordsets': 7, 'zone_records': 7,
            'recordset_records': 1, 'api_export_size': 7}
        self.adm_quota_client.set_quotas(
            project_id=tenant_id, quotas=quotas,
            headers=sudo_header)

        # Admin creates a zone for project "A" and share it with Primary
        zone = self._create_shared_zone_for_project(
            zone_name='test_recordset_records_enforced_against_owner',
            project_id=tenant_id, sudo_header=sudo_header)[0]

        # Primary creates recordset with (single record) --> Should PASS
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=zone['name'])
        body = self.rec_client.create_recordset(
            zone['id'], recordset_data)[1]
        self.addCleanup(
            self.wait_recordset_delete, self.rec_client,
            zone['id'], body['id'])

        # Primary creates one more recordset (two records) --> Should FAIL
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A', zone_name=zone['name'], number_of_records=2)
        with self.assertRaisesDns(
                lib_exc.OverLimit, 'over_quota', 413):
            self.rec_client.create_recordset(
                zone['id'], recordset_data)
